// hlavickove subory *******************************************************
#include "includes.h"

// konstruktor *************************************************************
CTexture::CTexture()
{
 // pocet nahratych textur
 tex_count=0;
 // pocet lightmap
 lm_count=0;

 // SDL nacitanie suboru s obrazkom textury
 img=NULL;
 // pomocna pamat
 data=NULL;
}

// destruktor **************************************************************
CTexture::~CTexture()
{
 // zrusenie objektov textur
 loopi(tex_count) {
  glDeleteTextures(1,&tex_id[i]);
  printf("zrusenie textury id: %u\n",tex_id[i]);
 }
 tex_count=0;
 // zrusenie objektov lightmap
 deleteLightmaps();
}

// uprava farieb bodov textury na zaklade typu suboru **********************
int CTexture::correctData(SDL_Surface *S)
{
 GLuint w=(GLuint) S->w,h=(GLuint) S->h;
 GLubyte *pixels=(GLubyte *) S->pixels;
 GLubyte palIndex;

 if (S->format==NULL) { printf("Chyba formatu obrazku!\n"); return(1); }

 // obrazok bez palety
 if (S->format->palette==NULL) {
  Bpp=S->format->BytesPerPixel; // pocet Bytov na pixel

  // INFO: Bpp
  //printf("Bpp: %u\n",Bpp);

  // vyhradenie pamate pre farby bodov textury
  data=new GLubyte[w*h*Bpp];
  if (data==NULL) { printf("Nemozno vyhradit pamat!\n"); return(1); }

  // prehodenie R a B zloziek farby?!
  // vertikalne otocenie obrazku - potrebne z dovodu, ze OpenGL textura
  // ma zaciatok v bode <0,0>, avsak obrazok ma v danom bode suradnicu <0,vyska>
  if (S->format->Bshift!=0) {
   loopj(h) {
    for (unsigned int i=0;i<w*3;i+=3) {
     data[i+j*w*Bpp+0]=pixels[i+(h-1-j)*w*Bpp+0]; // R - R?!
     data[i+j*w*Bpp+1]=pixels[i+(h-1-j)*w*Bpp+1]; // G - G
     data[i+j*w*Bpp+2]=pixels[i+(h-1-j)*w*Bpp+2]; // B - B?!
    }
   }
  }
  else { } // Bshift=0
 } // obrazok s paletou
 else {
  Bpp=3; // predpoklad 3 Byty na pixel

  // kontrola palety
  if (S->format->palette->colors==NULL) { printf("Chyba v palete obrazku!\n"); return(1); }

  // INFO: Bpp
  //printf("Bpp: %u\n",Bpp);

  // vyhradenie pamate pre farby bodov textury
  data=new GLubyte[w*h*Bpp];
  if (data==NULL) { printf("Nemozno vyhradit pamat!\n"); return(1); }

  unsigned int k;
  // kopirovanie R G B zloziek farby
  loopj(h) {
   k=0;
   loopi(w) {
    palIndex=pixels[i+(h-1-j)*w]; // index do palety
    data[k+j*w*Bpp+0]=S->format->palette->colors[palIndex].r; // R
    data[k+j*w*Bpp+1]=S->format->palette->colors[palIndex].g; // G
    data[k+j*w*Bpp+2]=S->format->palette->colors[palIndex].b; // B
    k+=3;
   }
  }
 }

 // vyber formatu udajov ulozenych v texture
 switch (Bpp) {
  case 1: glFormat=GL_ALPHA; break;
  case 3: glFormat=GL_RGB; break;
  case 4: glFormat=GL_RGBA; break;
  default: printf("Nemozno vybrat format udajov ulozenych v texture!\n"); return(1);
 }

 // vyber interneho formatu udajov ulozenych v texture
 switch (glFormat) {
  case GL_ALPHA:
   switch (Bpp) {
    case 1: intFormat=GL_ALPHA; break;
    case 4: intFormat=GL_ALPHA4; break;
    case 8: intFormat=GL_ALPHA8; break;
    case 12: intFormat=GL_ALPHA12; break;
    case 16: intFormat=GL_ALPHA16; break;
   }
  break;
  case GL_RGB:
   switch (Bpp) {
    case 2: intFormat=GL_R3_G3_B2; break;
    case 3: intFormat=GL_RGB; break;
    case 4: intFormat=GL_RGB4; break;
    case 5: intFormat=GL_RGB5; break;
    case 8: intFormat=GL_RGB8; break;
    case 10: intFormat=GL_RGB10; break;
    case 12: intFormat=GL_RGB12; break;
    case 16: intFormat=GL_RGB16; break;
   }
  break;
  case GL_RGBA:
   switch (Bpp) {
    case 2: intFormat=GL_RGBA2; break;
    case 4: intFormat=GL_RGBA4; break;
    case 5: intFormat=GL_RGB5_A1; break;
    case 8: intFormat=GL_RGBA8; break;
    case 10: intFormat=GL_RGB10_A2; break;
    case 12: intFormat=GL_RGBA12; break;
    case 16: intFormat=GL_RGBA16; break;
   }
  break;
 }

 return(0);
}

// nastavenie parametrov a vytvorenie 2D textury ***************************
void CTexture::genTexture(GLuint W,GLuint H,GLubyte *DATA,GLuint MODE)
{
 // nastavenie filteringu
 if (MODE&T_LINEAR) { // LINEAR
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  if (MODE&T_MIPMAP) {
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
  } else {
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    }
 } else {             // NEAREST
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
    if (MODE&T_MIPMAP) {
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST_MIPMAP_NEAREST);
    } else {
       glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
      }
   }
 // nastavenie opakovania sa textury
 if (MODE&T_CLAMP) {
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
 } else if (MODE&T_CLAMP_TO_EDGE) {
         glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
         glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
        } else {
           glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
           glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
          }
 // vytvorenie objektu textury
 if (MODE&T_MIPMAP) { // MIPMAP
  gluBuild2DMipmaps(GL_TEXTURE_2D,intFormat,W,H,glFormat,GL_UNSIGNED_BYTE,DATA);
 } else {             // standardna 2D textura
    glTexImage2D(GL_TEXTURE_2D,0,intFormat,W,H,0,glFormat,GL_UNSIGNED_BYTE,DATA);
   }
}

// nahratie 1D textury *****************************************************
int CTexture::load1D(GLuint ID,const char *F,GLuint MODE)
{
 if (ID>=MAX_TEX_NUM) {
  printf("Prekroceny maximalny pocet textur!\n"); return(1);
 }

 // nacitanie suboru s obrazkom textury
 img=IMG_Load(F); if (img==NULL) return(1);

 // vygenerovanie textury
 glGenTextures(1,&tex_id[ID]);
 tex_count++; // zvysenie poctu textur
 // pripojenie textury
 glBindTexture(GL_TEXTURE_1D,tex_id[ID]);

  // nastavenie filteringu
 if (MODE&T_LINEAR) { // LINEAR
  glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  if (MODE&T_MIPMAP) {
   glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
  } else {
     glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    }
 }
 else {               // NEAREST
  glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
  if (MODE&T_MIPMAP) {
   glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_MIN_FILTER,GL_NEAREST_MIPMAP_NEAREST);
  } else {
     glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
    }
 }
 // nastavenie opakovania sa textury
 if (MODE&T_CLAMP) {
  glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_WRAP_S,GL_CLAMP);
 } else if (MODE&T_CLAMP_TO_EDGE) {
         glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
        } else {
           glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_WRAP_S,GL_REPEAT);
          }
 // vytvorenie objektu textury
 if (MODE&T_MIPMAP) { // MIPMAP
  gluBuild1DMipmaps(GL_TEXTURE_1D,intFormat,img->w,glFormat,GL_UNSIGNED_BYTE,img->pixels);
 } else {             // standardna 1D textura
    glTexImage1D(GL_TEXTURE_1D,0,intFormat,img->w,0,glFormat,GL_UNSIGNED_BYTE,img->pixels);
   }

 // uvolnenie pouzitej pamate
 if (img) SDL_FreeSurface(img); img=NULL;

 // INFO: identifikator a nazov suboru textury
 printf("1D textura: %u %s\n",tex_id[ID],F);

 return(0);
}

// nahratie 2D textury *****************************************************
int CTexture::load2D(GLuint ID,const char *F,GLuint MODE)
{
 if (ID>=MAX_TEX_NUM) {
  printf("Prekroceny maximalny pocet textur!\n"); return(1);
 }

 // nacitanie suboru s obrazkom textury
 img=IMG_Load(F); if (img==NULL) return(1);

 // uprava farieb bodov textury na zaklade typu suboru
 if (correctData(img)) {
  printf("Nemozno upravit farby bodov textury!\n");
  // uvolnenie pouzitej pamate
  if (img) SDL_FreeSurface(img); img=NULL;
  return(1);
 }

 // vygenerovanie textury
 glGenTextures(1,&tex_id[ID]);
 tex_count++; // zvysenie poctu textur
 // pripojenie textury
 glBindTexture(GL_TEXTURE_2D,tex_id[ID]);
 // nastavenie parametrov a vytvorenie 2D textury
 genTexture((GLuint) img->w,(GLuint) img->h,data,MODE);

 // uvolnenie vyhradenej pamate (vyhradenej v correctData())
 DELETEA(data);

 // uvolnenie pouzitej pamate
 if (img) SDL_FreeSurface(img); img=NULL;

 // INFO: identifikator a nazov suboru textury
 printf("2D textura: %u %s\n",tex_id[ID],F);

 return(0);
}

// nahratie Bump textury ***************************************************
// zatial bez correctData()!
int CTexture::loadBump(GLuint ID1,GLuint ID2,const char *F,GLuint MODE)
{
 GLubyte *d; // iba ukazovatel na data (nevyhradi sa pamat)

 if ((ID1>=MAX_TEX_NUM)||(ID2>=MAX_TEX_NUM)) {
  printf("Prekroceny maximalny pocet textur!\n"); return(1);
 }

 // nacitanie suboru s obrazkom textury
 img=IMG_Load(F); if (img==NULL) return(1);

 glPixelTransferf(GL_RED_SCALE,  0.5f); // 50% intenzita
 glPixelTransferf(GL_GREEN_SCALE,0.5f);
 glPixelTransferf(GL_BLUE_SCALE, 0.5f);

 // vygenerovanie textury
 glGenTextures(1,&tex_id[ID1]);
 tex_count++; // zvysenie poctu textur
 // pripojenie textury
 glBindTexture(GL_TEXTURE_2D,tex_id[ID1]);
 // nastavenie parametrov a vytvorenie 2D textury
 genTexture((GLuint) img->w,(GLuint) img->h,(GLubyte *) img->pixels,MODE);

 // vytvorenie invertovanej textury
 d=(GLubyte *) img->pixels;
 loopi(3*img->w*img->h) d[i]=255-d[i];

 // vygenerovanie textury
 glGenTextures(1,&tex_id[ID2]);
 tex_count++; // zvysenie poctu textur
 // pripojenie textury
 glBindTexture(GL_TEXTURE_2D,tex_id[ID2]);
 // nastavenie parametrov a vytvorenie 2D textury
 genTexture((GLuint) img->w,(GLuint) img->h,d,MODE);

 glPixelTransferf(GL_RED_SCALE,  1.0f); // 100% intenzita
 glPixelTransferf(GL_GREEN_SCALE,1.0f);
 glPixelTransferf(GL_BLUE_SCALE, 1.0f);

 // uvolnenie pouzitej pamate
 if (img) SDL_FreeSurface(img); img=NULL;

 // INFO: identifikator a nazov suboru textury
 printf("Bump1 textura: %u %s\n",tex_id[ID1],F);
 // INFO: identifikator a nazov suboru textury
 printf("Bump2 textura: %u %s\n",tex_id[ID2],F);

 return(0);
}

// nahratie CubeMap textury ************************************************
// zatial bez correctData()!
int CTexture::loadCubemap(GLuint ID,
                          const char *F1,const char *F2,const char *F3,
                          const char *F4,const char *F5,const char *F6,GLuint MODE)
{
 if (ID>=MAX_TEX_NUM) {
  printf("Prekroceny maximalny pocet textur!\n"); return(1);
 }

 // vygenerovanie textury
 glGenTextures(1,&tex_id[ID]);
 tex_count++; // zvysenie poctu textur
 // pripojenie textury
 glBindTexture(GL_TEXTURE_CUBE_MAP_ARB,tex_id[ID]);
 // nastavenie filteringu
 glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
 glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
 // nastavenie opakovania sa textury
 if (MODE&T_CLAMP_TO_EDGE) {
  glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
 } else {
    glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_WRAP_S,GL_REPEAT);
    glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_WRAP_T,GL_REPEAT);
   }

 // vlozenie obrazkov do textury
 loopi(6) {
  switch(i) {
   case 0: img=IMG_Load(F1); if (img==NULL) return(1);
           // vytvorenie objektu textury
           glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB,0,GL_RGB,img->w,img->h,
                        0,GL_BGR,GL_UNSIGNED_BYTE,img->pixels);
           // uvolnenie pouzitej pamate
           if (img) SDL_FreeSurface(img); img=NULL;
           // INFO: identifikator a nazov suboru textury
           printf("Cubemap1 textura: %u %s\n",i,F1);
   break;
   case 1: img=IMG_Load(F2); if (img==NULL) return(1);
           // vytvorenie objektu textury
           glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB,0,GL_RGB,img->w,img->h,
                        0,GL_BGR,GL_UNSIGNED_BYTE,img->pixels);
           // uvolnenie pouzitej pamate
           if (img) SDL_FreeSurface(img); img=NULL;
           // INFO: identifikator a nazov suboru textury
           printf("Cubemap2 textura: %u %s\n",i,F2);
   break;
   case 2: img=IMG_Load(F3); if (img==NULL) return(1);
           // vytvorenie objektu textury
           glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB,0,GL_RGB,img->w,img->h,
                        0,GL_BGR,GL_UNSIGNED_BYTE,img->pixels);
           // uvolnenie pouzitej pamate
           if (img) SDL_FreeSurface(img); img=NULL;
           // INFO: identifikator a nazov suboru textury
           printf("Cubemap3 textura: %u %s\n",i,F3);
   break;
   case 3: img=IMG_Load(F4); if (img==NULL) return(1);
           // vytvorenie objektu textury
           glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB,0,GL_RGB,img->w,img->h,
                        0,GL_BGR,GL_UNSIGNED_BYTE,img->pixels);
           // uvolnenie pouzitej pamate
           if (img) SDL_FreeSurface(img); img=NULL;
           // INFO: identifikator a nazov suboru textury
           printf("Cubemap4 textura: %u %s\n",i,F4);
   break;
   case 4: img=IMG_Load(F5); if (img==NULL) return(1);
           // vytvorenie objektu textury
           glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB,0,GL_RGB,img->w,img->h,
                        0,GL_BGR,GL_UNSIGNED_BYTE,img->pixels);
           // uvolnenie pouzitej pamate
           if (img) SDL_FreeSurface(img); img=NULL;
           // INFO: identifikator a nazov suboru textury
           printf("Cubemap5 textura: %u %s\n",i,F5);
   break;
   case 5: img=IMG_Load(F6); if (img==NULL) return(1);
           // vytvorenie objektu textury
           glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB,0,GL_RGB,img->w,img->h,
                        0,GL_BGR,GL_UNSIGNED_BYTE, img->pixels);
           // uvolnenie pouzitej pamate
           if (img) SDL_FreeSurface(img); img=NULL;
           // INFO: identifikator a nazov suboru textury
           printf("Cubemap6 textura: %u %s\n",i,F6);
   break;
  }
 }

 return(0);
}

// zrusenie OpenGL textury *************************************************
void CTexture::freeTexture(GLuint ID)
{
 glDeleteTextures(1,&tex_id[ID]);
}

// vytvorenie lightmapy ****************************************************
int CTexture::createLightmap(GLuint ID)
{
 if (ID>=MAX_LM_NUM) {
  printf("Prekroceny maximalny pocet lightmap!\n"); return(1);
 }

 // lumel = luminance element = bod lightmapy

 // vyhradenie pamate pre odtien lumelov lightmapy
 data=new GLubyte[LM_SIZE*LM_SIZE]; // 1 Byte / lumel
 if (data==NULL) { printf("Nemozno vyhradit pamat!\n"); return(1); }
 memset(data,LM_DIP,LM_SIZE*LM_SIZE);

 // vygenerovanie textury
 glGenTextures(1,&lm_id[ID]);
 lm_count++; // zvysenie poctu lightmap
 // pripojenie textury
 glBindTexture(GL_TEXTURE_2D,lm_id[ID]);
 // nastavenie parametrov a vytvorenie 2D textury
 // nastavenie filteringu
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
 // nastavenie opakovania sa textury
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
 // vytvorenie objektu textury
 glTexImage2D(GL_TEXTURE_2D,0,GL_LUMINANCE,LM_SIZE,LM_SIZE,
              0,GL_LUMINANCE,GL_UNSIGNED_BYTE,data);

 // uvolnenie vyhradenej pamate
 DELETEA(data);

 // INFO: identifikator lightmapy
 printf("Lightmapa: %u\n",lm_id[ID]);

 return(0);
}

// zrusenie objektov lightmap **********************************************
void CTexture::deleteLightmaps(void)
{
 // zrusenie objektov lightmap
 loopi(lm_count) {
  glDeleteTextures(1,&lm_id[i]);
  printf("zrusenie Lightmapy id: %u\n",lm_id[i]);
 }
 lm_count=0;
}
