#ifndef _TEXTURE_H_
#define _TEXTURE_H_

// maximalny pocet textur
#define MAX_TEX_NUM 256
// maximalny pocet lightmap
#define MAX_LM_NUM 256

// sirka (vyska) lightmapy
#define LM_SIZE 512
// sirka (vyska) segmentu lightmapy priradenej trojuholniku/stvoruholniku
#define LM_SEGMENT 64
// standardny odtien bodov lightmapy
#define LM_DIP 64

// volitelne parametre textury
enum {
 T_DEFAULT=1<<0,
 T_CLAMP=1<<1,
 T_CLAMP_TO_EDGE=1<<2,
 T_NEAREST=1<<3,
 T_LINEAR=1<<4,
 T_MIPMAP=1<<5
};

// trieda premennych a funkcii (objektov) pre pracu s OpenGL texturami
class CTexture {
public:
 CTexture();  // konstruktor
 ~CTexture(); // destruktor

 GLuint tex_count; // pocet nahratych textur
 GLuint lm_count;  // pocet lightmap

 GLuint tex_id[MAX_TEX_NUM]; // identifikatory nahratych textur
 GLuint lm_id[MAX_LM_NUM];   // identifikatory lightmap

 int load1D(GLuint ID,const char *F,GLuint MODE);               // nahratie 1D textury
 int load2D(GLuint ID,const char *F,GLuint MODE);               // nahratie 2D textury
 int loadBump(GLuint ID1,GLuint ID2,const char *F,GLuint MODE); // nahratie Bump textury
 int loadCubemap(GLuint ID,
                 const char *F1,const char *F2,const char *F3,  // nahratie Cubemap textury
                 const char *F4,const char *F5,const char *F6,GLuint MODE);

 void freeTexture(GLuint ID); // zrusenie OpenGL textury

 int createLightmap(GLuint ID); // vytvorenie lightmapy
 void deleteLightmaps(void);    // zrusenie objektov lightmap
private:
 // SDL nacitanie suboru s obrazkom textury
 SDL_Surface *img;

 // pomocna pamat
 GLubyte *data;

 // pocet Bytov na pixel (bod textury)
 GLuint Bpp;
 // interny format a format udajov ulozenych v texture (na zaklade Bpp)
 GLuint intFormat,glFormat;

 // uprava farieb bodov textury na zaklade typu suboru
 int correctData(SDL_Surface *S);
 // nastavenie parametrov a vytvorenie 2D textury
 void genTexture(GLuint W,GLuint H,GLubyte *DATA,GLuint MODE);
};

#endif
